'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Hypothetical Primitive
 *
 * NOTE: This is out of band for our implementation, because there are
 * no pointers in JS and it doesn't really help us with JSON Document classes.
 * Initializing primitives *is* in scope for the specification.
 *
 * {
 *   type: 'number', default: 1337
 * }
 */
function hypotheticalPrimitive(target, source, options) {
  if (options.default !== false) {
    target = options.default;
  }

  target = source;

  return target; // blech
}

/**
 * Simple Object
 *
 * {
 *   properties: {
 *    a: { type: 'number' },
 *    b: { type: 'string' }
 *   }
 * }
 */
function simpleObject(target, source, options) {
  if (!source || (typeof source === 'undefined' ? 'undefined' : _typeof(source)) !== 'object') {
    source = {};
  }

  if (!target || (typeof target === 'undefined' ? 'undefined' : _typeof(target)) !== 'object') {
    throw new Error('Target must be an object');
  }

  options = options || {};

  // root-level declaration
  var target0 = target;
  var source0 = source;
  var count0 = 0;

  // sourceHasOwnProperty
  if (source0.hasOwnProperty('a')) {
    // assignTargetValueFromSource
    target0['a'] = source0['a'];
    // incrementCounter
    count0++;
  }

  // sourceHasOwnProperty
  if (source0.hasOwnProperty('b')) {
    // assignTargetValueFromSource
    target0['b'] = source0['b'];
    // incrementCounter
    count0++;
  }
}

/**
 * Simple Array
 *
 * {
 *   type: 'array',
 *   items: {
 *     type: 'object',
 *     properties: {
 *       foo: { default: 'bar' }
 *     }
 *   }
 * }
 *
 * [{ foo: 'bar' }]
 *
 */
function simpleArray(target, source, options) {
  if (!source || !Array.isArray(source)) {
    source = [];
  }

  //
}

/**
 * {
 *   type: 'object',
 *   properties: {
 *     a: {
 *       type: 'array',
 *       items: [
 *         {
 *          type: 'object',
 *          properties: {
 *            b: { type: 'number' }
 *          }
 *        }
 *       ]
 *     }
 *   }
 * }
 */
function wtf(target, source, options) {}

/**
 * nestedObject
 *
 * {
 *   type: 'object',
 *   properties: {
 *     a: {
 *       type: 'object',
 *       properties: {
 *         b: { type: 'string' }
 *       }
 *     }
 *   }
 * }
 *
 */

function nestedObject(target, source, options) {
  if (!source || (typeof source === 'undefined' ? 'undefined' : _typeof(source)) !== 'object') {
    source = {};
  }

  if (!target || (typeof target === 'undefined' ? 'undefined' : _typeof(target)) !== 'object') {
    throw new Error('Target must be an object');
  }

  options = options || {};

  // root-level declaration
  var target0 = target;
  var source0 = source;
  var count0 = 0;

  // level 1 declaration
  var target1;
  var source1;
  var count1;

  // sourceHasOwnProperty
  if (source0.hasOwnProperty('a')) {
    // sourceValueIsObject
    if (_typeof(source0['a']) === 'object') {
      // ensureNestedTargetContainer
      //  testTargetParentHasOwnProperty        testTargetIsObject
      if (!target0.hasOwnProperty('a') || _typeof(target0['a']) !== 'object') {
        // assignNewTargetContainerObject
        target1 = {};
      } else {
        // assignExistingTargetContainerObject
        target1 = target0['a'];
      }

      // assignNestedSourceContainer
      source1 = source0['a'];
      count1 = 0;

      // sourceHasOwnProperty
      if (source1.hasOwnProperty('b')) {
        // assignTargetValueFromSource
        target1['b'] = source1['b'];
        // incrementCounter
        count1++;
      }

      if (count1 > 0) {
        target0['a'] = target1;
        count0++;
      }

      // sourceValueIsNotExpectedType
    } else {
      target0['a'] = source0['a'];
      count0++;
    }
  }
}

/**
 *
 * {
 *   type: 'object',
 *   properties: {
 *     a: {
 *       type: 'array',
 *       items: { type: 'string' }
 *     }
 *   }
 * }
 *
 */
{
  a: ['value'];
}

/**
 *
 * {
 *   type: 'object',
 *   properties: {
 *     a: {
 *       type: 'array',
 *       items: {
 *         type: 'object',
 *         properties: {
 *           b: { type: 'string' }
 *         }
 *       }
 *     }
 *   }
 * }
 *
 */
{
  a: [{ b: 'value' }];
}

/**
 *
 * {
 *   type: 'object',
 *   properties: {
 *     a: {
 *       type: 'array',
 *       items: {
 *         type: 'array',
 *         items: { type: 'string' }
 *       }
 *     }
 *   }
 * }
 *
 */
{
  a: [['value']];
}